<project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 http://maven.apache.org/maven-v4_0_0.xsd">
    <modelVersion>4.0.0</modelVersion>
    <groupId>FIXME</groupId>
    <artifactId>FIXME</artifactId>
    <version>FIXME</version>
    <packaging>pom</packaging>

    <dependencies>
        $dependencies
    </dependencies>

    <repositories>
        $repositories 
    </repositories>
</project>    
